from pathlib import Path
from pyfonycore import pyproject

def isDefined(rawConfig):
    return 'pyfony' in rawConfig and 'bootstrap' in rawConfig['pyfony']

def read(pyprojectPath: Path):
    config = pyproject.read(pyprojectPath)
    return getBoostrapConfig(config)

def getBoostrapConfig(rawConfig):
    if 'pyfony' not in rawConfig:
        raise Exception('[pyfony] section is missing in pyproject.toml')

    if 'bootstrap' not in rawConfig['pyfony']:
        raise Exception('[pyfony.bootstrap] section is missing in pyproject.toml')

    return rawConfig['pyfony']['bootstrap']



from pyfonycore.bootstrap.config.Config import Config
from pyfonycore.bootstrap.config.raw import containerInitResolver, rootModuleNameResolver, allowedEnvironments

def create(rawConfig, pyprojectSource: str):
    if containerInitResolver.containerInitDefined(rawConfig):
        initContainer = containerInitResolver.resolve(rawConfig, pyprojectSource)
    else:
        from pyfonycore.container.containerInit import initContainer # pylint: disable = import-outside-toplevel

    return Config(
        initContainer,
        rootModuleNameResolver.resolve(rawConfig, pyprojectSource),
        allowedEnvironments.get(rawConfig)
    )



import unittest
from injecta.config.YamlConfigReader import YamlConfigReader
from injecta.mocks.Bar import Bar
from injecta.mocks.Foo import Foo
from injecta.package.pathResolver import resolvePath
from pyfonycore.Kernel import Kernel

class KernelTest(unittest.TestCase):

    def test_basic(self):
        kernel = Kernel(
            'test',
            resolvePath('pyfonycore') + '/_config',
            YamlConfigReader()
        )

        container = kernel.initContainer()

        foo = container.get(Foo)
        bar = container.get('injecta.mocks.Bar')

        self.assertIsInstance(foo, Foo)
        self.assertIsInstance(bar, Bar)

if __name__ == '__main__':
    unittest.main()



class Config:

    def __init__(
        self,
        containerInitFunction: callable,
        rootModuleName: str,
        allowedEnvironments: list,
    ):
        self.__containerInitFunction = containerInitFunction
        self.__rootModuleName = rootModuleName
        self.__allowedEnvironments = allowedEnvironments

    @property
    def containerInitFunction(self):
        return self.__containerInitFunction

    @property
    def rootModuleName(self):
        return self.__rootModuleName

    @property
    def allowedEnvironments(self):
        return self.__allowedEnvironments



import os
from typing import List
from injecta.config.ConfigReaderInterface import ConfigReaderInterface
from injecta.container.ContainerBuild import ContainerBuild
from injecta.container.ContainerBuilder import ContainerBuilder
from injecta.container.ContainerInitializer import ContainerInitializer
from injecta.container.ContainerInterface import ContainerInterface
from pyfonybundles.Bundle import Bundle
from pyfonycore.container.PyfonyHook import PyfonyHooks

class Kernel:

    _allowedEnvironments = ['dev', 'test', 'prod']

    def __init__(
        self,
        appEnv: str,
        configDir: str,
        configReader: ConfigReaderInterface,
        bundles: List[Bundle] = None,
    ):
        self._appEnv = appEnv
        self._configDir = configDir
        self.__configReader = configReader
        self._bundles = bundles or []
        self._containerBuilder = ContainerBuilder()

    def setAllowedEnvironments(self, allowedEnvironments: list):
        self._allowedEnvironments = allowedEnvironments

    def initContainer(self) -> ContainerInterface:
        if self._appEnv not in self._allowedEnvironments:
            raise Exception(f'Unexpected environment: {self._appEnv}')

        hooks = self._createPyfonyHooks()
        return self._initContainerFromHooks(hooks)

    def _initContainerFromHooks(self, hooks: PyfonyHooks):
        config = self.__configReader.read(self._getConfigPath())
        containerBuild = self._containerBuilder.build(config, hooks)
        return self._initAndBootContainer(containerBuild)

    def _createPyfonyHooks(self):
        return PyfonyHooks(
            self._bundles,
            self._getConfigPath(),
            self._getProjectBundlesConfigDir(),
            self._appEnv
        )

    def _initAndBootContainer(self, containerBuild: ContainerBuild):
        container = ContainerInitializer().init(containerBuild)
        self._boot(container)

        return container

    def _getConfigPath(self):
        return f'{self._configDir}/config_{self._appEnv}.yaml'

    def _getProjectBundlesConfigDir(self):
        return os.path.dirname(self._getConfigPath()) + '/bundles'

    def _boot(self, container: ContainerInterface):
        for bundle in self._bundles:
            bundle.boot(container)



import os
from typing import List
from box import Box
from injecta.compiler.CompilerPassInterface import CompilerPassInterface
from injecta.config.ConfigMerger import ConfigMerger
from injecta.container.ContainerBuild import ContainerBuild
from injecta.container.Hooks import Hooks
from injecta.service.Service import Service
from pyfonybundles.Bundle import Bundle
from pyfonybundles.BundleManager import BundleManager

class PyfonyHooks(Hooks):

    def __init__(
        self,
        bundles: List[Bundle],
        configPath: str,
        projectBundlesConfigDir: str,
        appEnv: str
    ):
        self.__configMerger = ConfigMerger()
        self.__bundleManager = BundleManager(bundles)
        self.__configPath = configPath
        self.__projectBundlesConfigDir = projectBundlesConfigDir
        self.__appEnv = appEnv

    def start(self, rawConfig: dict) -> dict:
        bundlesConfig = self.__bundleManager.getBundlesConfig()
        projectBundlesConfig = self.__bundleManager.getProjectBundlesConfig(self.__projectBundlesConfigDir)

        rawConfig = self.__configMerger.merge(
            self.__configMerger.merge(bundlesConfig, projectBundlesConfig),
            rawConfig
        )

        return self.__bundleManager.modifyRawConfig(rawConfig)

    def servicesPrepared(self, services: List[Service]) -> List[Service]:
        return self.__bundleManager.modifyServices(services)

    def getCustomParameters(self) -> dict:
        pyfonyCustomParameters = {
            'project': {
                'configDir': os.path.dirname(self.__configPath),
            },
            'kernel': {
                'environment': self.__appEnv,
            },
        }

        return self.__configMerger.merge(super().getCustomParameters(), pyfonyCustomParameters, False)

    def parametersParsed(self, parameters: Box) -> Box:
        return self.__bundleManager.modifyParameters(parameters)

    def containerBuildReady(self, containerBuild: ContainerBuild):
        for compilerPass in self.__bundleManager.getCompilerPasses(): # type: CompilerPassInterface
            compilerPass.process(containerBuild)



from injecta.config.YamlConfigReader import YamlConfigReader
from injecta.container.ContainerInterface import ContainerInterface
from injecta.package.pathResolver import resolvePath
from pyfonycore.Kernel import Kernel
from pyfonybundles.loader import pyfonyBundlesLoader
from pyfonycore.bootstrap.config.Config import Config

def initContainer(appEnv: str, bootstrapConfig: Config) -> ContainerInterface:
    bundles = pyfonyBundlesLoader.loadBundles()

    kernel = Kernel(
        appEnv,
        resolvePath(bootstrapConfig.rootModuleName) + '/_config',
        YamlConfigReader(),
        bundles,
    )
    kernel.setAllowedEnvironments(bootstrapConfig.allowedEnvironments)

    return kernel.initContainer()



from pyfonycore.bootstrap.config.raw import containerInitResolver

def resolve(rawConfig, pyprojectSource: str):
    if 'rootModuleName' in rawConfig:
        return rawConfig['rootModuleName']

    containerInitSpec = containerInitResolver.getContainerInit(rawConfig, pyprojectSource)

    return _resolveRootModuleName(containerInitSpec[0])

def _resolveRootModuleName(moduleName):
    return moduleName[0:moduleName.find('.')]



from pyfonycore import pyproject
from pyfonycore.bootstrap.config import configFactory
from pyfonycore.bootstrap.config.Config import Config
from pyfonycore.bootstrap.config.raw import rawConfigReader

def read() -> Config:
    rawConfig = rawConfigReader.read(pyproject.getPath())
    return configFactory.create(rawConfig, '[pyfony.bootstrap]')



from injecta.dtype.classLoader import loadClass

def containerInitDefined(rawConfig):
    return 'containerInit' in rawConfig

def resolve(rawConfig, pyprojectSource: str):
    containerInitFunctionSpec = getContainerInit(rawConfig, pyprojectSource)
    return loadClass(*containerInitFunctionSpec)

def getContainerInit(rawConfig: dict, pyprojectSource: str):
    if 'containerInit' not in rawConfig:
        raise Exception(f'containerInit is missing in {pyprojectSource} in pyproject.toml')

    return rawConfig['containerInit'].split(':')



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pyfonycore',
 'pyfonycore.bootstrap.config',
 'pyfonycore.bootstrap.config.raw',
 'pyfonycore.container']

package_data = \
{'': ['*'], 'pyfonycore': ['_config/*']}

install_requires = \
['injecta>=0.8.11,<0.9.0', 'pyfony-bundles==0.2.5a2']

setup_kwargs = {
    'name': 'pyfony-core',
    'version': '0.7.0a4',
    'description': 'Pyfony core',
    'long_description': '# Pyfony Core\n\nPart of the [Pyfony Framewok](https://github.com/pyfony/pyfony)\n\n## Installation\n\n```\n$ poetry add pyfony-core\n```\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyfony/core',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<3.8.0',
}


setup(**setup_kwargs)
